# -*- encoding: utf-8 -*-
from abjad import *


def test_sequencetools_partition_sequence_by_weights_at_least_01():

    sequence = [3, 3, 3, 3, 4, 4, 4, 4, 5, 5]
    groups = sequencetools.partition_sequence_by_weights_at_least(
        sequence, [10, 4], cyclic=False, overhang=True)
    assert groups == [[3, 3, 3, 3], [4], [4, 4, 4, 5, 5]]


def test_sequencetools_partition_sequence_by_weights_at_least_02():

    sequence = [3, 3, 3, 3, 4, 4, 4, 4, 5, 5]
    groups = sequencetools.partition_sequence_by_weights_at_least(
        sequence, [10, 4], cyclic=False, overhang=False)
    assert groups == [[3, 3, 3, 3], [4]]


def test_sequencetools_partition_sequence_by_weights_at_least_03():

    sequence = [3, 3, 3, 3, 4, 4, 4, 4, 5, 5]
    groups = sequencetools.partition_sequence_by_weights_at_least(
        sequence, [10, 4], cyclic=True, overhang=True)
    assert groups == [[3, 3, 3, 3], [4], [4, 4, 4], [5], [5]]


def test_sequencetools_partition_sequence_by_weights_at_least_04():

    sequence = [3, 3, 3, 3, 4, 4, 4, 4, 5, 5]
    groups = sequencetools.partition_sequence_by_weights_at_least(
        sequence, [10, 4], cyclic=True, overhang=False)
    assert groups == [[3, 3, 3, 3], [4], [4, 4, 4], [5]]



# -*- encoding: utf-8 -*-
from abjad.tools import markuptools
from abjad.tools import pitchtools
from abjad.tools import scoretools
from abjad.tools.topleveltools import attach
from abjad.tools.topleveltools import iterate


def label_vertical_moments_in_expr_with_named_intervals(
    expr, markup_direction=Down):
    r'''Label named intervals of every vertical moment in `expr`:

    ::

        >>> score = Score([])
        >>> staff = Staff("c'8 d'8 e'8 f'8")
        >>> score.append(staff)
        >>> staff = Staff(r"""\clef "alto" g4 f4""")
        >>> score.append(staff)
        >>> staff = Staff(r"""\clef "bass" c,2""")
        >>> score.append(staff)

    ::

        >>> labeltools.label_vertical_moments_in_expr_with_named_intervals(score)

    ..  doctest::

        >>> print format(score)
        \new Score <<
            \new Staff {
                c'8
                d'8
                    _ \markup {
                        \small
                            \column
                                {
                                    16
                                    12
                                }
                        }
                e'8
                f'8
                    _ \markup {
                        \small
                            \column
                                {
                                    18
                                    11
                                }
                        }
            }
            \new Staff {
                \clef "alto"
                g4
                f4
                    _ \markup {
                        \small
                            \column
                                {
                                    17
                                    11
                                }
                        }
            }
            \new Staff {
                \clef "bass"
                c,2
                    _ \markup {
                        \small
                            \column
                                {
                                    15
                                    12
                                }
                        }
            }
        >>

    ::

        >>> show(score) # doctest: +SKIP

    Returns none.
    '''

    for vertical_moment in \
        iterate(expr).by_vertical_moment():
        leaves = vertical_moment.leaves
        notes = [leaf for leaf in leaves if isinstance(leaf, scoretools.Note)]
        if not notes:
            continue
        notes.sort(key=lambda x: x.written_pitch.numbered_pitch)
        notes.reverse()
        bass_note = notes[-1]
        upper_notes = notes[:-1]
        named_intervals = []
        for upper_note in upper_notes:
            named_interval = pitchtools.NamedInterval.from_pitch_carriers(
                bass_note.written_pitch, upper_note.written_pitch)
            named_intervals.append(named_interval)
        intervals = [x.number for x in named_intervals]
        intervals = ' '.join([str(x) for x in intervals])
        intervals = r'\small \column { %s }' % intervals
        markup = markuptools.Markup(intervals, markup_direction)
        attach(markup, vertical_moment.start_leaves[-1])



# -*- encoding: utf-8 -*-
from abjad import *


def test_pitchtools_clef_and_staff_position_number_to_named_pitch_01():

    clef = Clef('treble')

    pitch = pitchtools.clef_and_staff_position_number_to_named_pitch(clef, -1)
    assert pitch == NamedPitch('a', 4)

    pitch = pitchtools.clef_and_staff_position_number_to_named_pitch(clef, 0)
    assert pitch == NamedPitch('b', 4)

    pitch = pitchtools.clef_and_staff_position_number_to_named_pitch(clef, 1)
    assert pitch == NamedPitch('c', 5)


def test_pitchtools_clef_and_staff_position_number_to_named_pitch_02():

    clef = Clef('alto')

    pitch = pitchtools.clef_and_staff_position_number_to_named_pitch(clef, -1)
    assert pitch == NamedPitch('b', 3)

    pitch = pitchtools.clef_and_staff_position_number_to_named_pitch(clef, 0)
    assert pitch == NamedPitch('c', 4)

    pitch = pitchtools.clef_and_staff_position_number_to_named_pitch(clef, 1)
    assert pitch == NamedPitch('d', 4)


def test_pitchtools_clef_and_staff_position_number_to_named_pitch_03():

    clef = Clef('bass')

    pitch = pitchtools.clef_and_staff_position_number_to_named_pitch(clef, -1)
    assert pitch == NamedPitch('c', 3)

    pitch = pitchtools.clef_and_staff_position_number_to_named_pitch(clef, 0)
    assert pitch == NamedPitch('d', 3)

    pitch = pitchtools.clef_and_staff_position_number_to_named_pitch(clef, 1)
    assert pitch == NamedPitch('e', 3)



# -*- encoding: utf-8 -*-
from abjad.tools import mathtools


def yield_all_restricted_growth_functions_of_length(length):
    '''Generate all restricted growth functions of `length` in lex order:

    ::

        >>> for rgf in sequencetools.yield_all_restricted_growth_functions_of_length(4):
        ...     rgf
        ...
        (1, 1, 1, 1)
        (1, 1, 1, 2)
        (1, 1, 2, 1)
        (1, 1, 2, 2)
        (1, 1, 2, 3)
        (1, 2, 1, 1)
        (1, 2, 1, 2)
        (1, 2, 1, 3)
        (1, 2, 2, 1)
        (1, 2, 2, 2)
        (1, 2, 2, 3)
        (1, 2, 3, 1)
        (1, 2, 3, 2)
        (1, 2, 3, 3)
        (1, 2, 3, 4)

    Returns generator of tuples.
    '''

    if not mathtools.is_positive_integer(length):
        raise TypeError

    last_rgf = range(1, length + 1)

    rgf = length * [1]
    yield tuple(rgf)

    while not rgf == last_rgf:
        for i, x in enumerate(reversed(rgf)):
            if x < max(rgf[:-(i+1)]) + 1:
                first_part = rgf[:-(i+1)]
                increased_part = [rgf[-(i+1)] + 1]
                trailing_ones = i * [1]
                rgf = first_part + increased_part + trailing_ones
                yield tuple(rgf)
                break



# -*- encoding: utf-8 -*-
from abjad import *


def test_pitchtools_NamedPitch___abs___01():

    assert abs(NamedPitch(11)) == 11
    assert abs(NamedPitch(11.5)) == 11.5
    assert abs(NamedPitch(13)) == 13
    assert abs(NamedPitch(13.5)) == 13.5



# -*- encoding: utf-8 -*-


def all_are_pairs(expr):
    r'''True when `expr` is a sequence whose members are all sequences of length 2:

    ::

        >>> sequencetools.all_are_pairs([(1, 2), (3, 4), (5, 6), (7, 8)])
        True

    True when `expr` is an empty sequence:

    ::

        >>> sequencetools.all_are_pairs([])
        True

    Otherwise false:

    ::

        >>> sequencetools.all_are_pairs('foo')
        False

    Returns boolean.
    '''

    try:
        return all(len(x) == 2 for x in expr)
    except TypeError:
        return False



# -*- encoding: utf-8 -*-
from abjad.tools.sequencetools.iterate_sequence_pairwise_strict \
	import iterate_sequence_pairwise_strict


def count_length_two_runs_in_sequence(sequence):
    '''Count length-``2`` runs in `sequence`:

    ::

        >>> sequencetools.count_length_two_runs_in_sequence([0, 0, 1, 1, 1, 2, 3, 4, 5])
        3

    Returns nonnegative integer.
    '''

    total_repetitions = 0
    for left, right in iterate_sequence_pairwise_strict(sequence):
        if left == right:
            total_repetitions += 1

    return total_repetitions



# -*- encoding: utf-8 -*-
from abjad import *


def test_sequencetools_sum_consecutive_sequence_elements_by_sign_01():

    sequence_1 = [0, 0, -1, -1, 2, 3, -5, 1, 2, 5, -5, -6]

    sequence_2 = list(sequencetools.sum_consecutive_sequence_elements_by_sign(sequence_1))
    assert sequence_2 == [0, -2, 5, -5, 8, -11]

    sequence_2 = list(sequencetools.sum_consecutive_sequence_elements_by_sign(sequence_1, sign=[-1]))
    assert sequence_2 == [0, 0, -2, 2, 3, -5, 1, 2, 5, -11]

    sequence_2 = list(sequencetools.sum_consecutive_sequence_elements_by_sign(sequence_1, sign=[0]))
    assert sequence_2 == [0, -1, -1, 2, 3, -5, 1, 2, 5, -5, -6]

    sequence_2 = list(sequencetools.sum_consecutive_sequence_elements_by_sign(sequence_1, sign=[1]))
    assert sequence_2 == [0, 0, -1, -1, 5, -5, 8, -5, -6]

    sequence_2 = list(sequencetools.sum_consecutive_sequence_elements_by_sign(sequence_1, sign=[-1, 0]))
    assert sequence_2 == [0, -2, 2, 3, -5, 1, 2, 5, -11]

    sequence_2 = list(sequencetools.sum_consecutive_sequence_elements_by_sign(sequence_1, sign=[-1, 1]))
    assert sequence_2 == [0, 0, -2, 5, -5, 8, -11]

    sequence_2 = list(sequencetools.sum_consecutive_sequence_elements_by_sign(sequence_1, sign=[0, 1]))
    assert sequence_2 == [0, -1, -1, 5, -5, 8, -5, -6]

    sequence_2 = list(sequencetools.sum_consecutive_sequence_elements_by_sign(sequence_1, sign=[-1, 0, 1]))
    assert sequence_2 == [0, -2, 5, -5, 8, -11]



# -*- encoding: utf-8 -*-
from abjad import *


def test_markuptools_MarkupCommand__get_format_pieces_01():

    circle = markuptools.MarkupCommand('draw-circle', 2.5, 0.1, False)
    square = markuptools.MarkupCommand('rounded-box', 'hello?')
    line = markuptools.MarkupCommand('line', [square, 'wow!'])
    rotate = markuptools.MarkupCommand('rotate', 60, line)
    combine = markuptools.MarkupCommand('combine', rotate, circle)

    assert combine._get_format_pieces() == [
        '\\combine',
        '\t\\rotate',
        '\t\t#60',
        '\t\t\\line',
        '\t\t\t{',
        '\t\t\t\t\\rounded-box',
        '\t\t\t\t\thello?',
        '\t\t\t\twow!',
        '\t\t\t}',
        '\t\\draw-circle',
        '\t\t#2.5',
        '\t\t#0.1',
        '\t\t##f'
    ]



# -*- encoding: utf-8 -*-
import pytest
from abjad import *


def test_agenttools_InspectionAgent_get_spanner_01():

    container = Container("c'8 d'8 e'8 f'8")
    beam = Beam()
    attach(beam, container.select_leaves()[:-1])
    slur = Slur()
    attach(slur, container.select_leaves()[:-1])
    trill = spannertools.TrillSpanner()
    attach(trill, container)

    assert systemtools.TestManager.compare(
        container,
        r'''
        {
            c'8 [ ( \startTrillSpan
            d'8
            e'8 ] )
            f'8 \stopTrillSpan
        }
        '''
        )

    assert inspect(container).get_spanner() == trill

    string = 'inspect(container[0]).get_spanner()'
    assert pytest.raises(Exception, string)

    string = 'inspect(container[-1]).get_spanner()'
    assert pytest.raises(Exception, string)



# -*- encoding: utf-8 -*-
from abjad.tools import scoretools


def make_basic_lilypond_file(music=None):
    r'''Make basic LilyPond file with `music`:

    ::

        >>> score = Score([Staff("c'8 d'8 e'8 f'8")])
        >>> lilypond_file = lilypondfiletools.make_basic_lilypond_file(score)
        >>> lilypond_file.header_block.composer = markuptools.Markup('Josquin')
        >>> lilypond_file.layout_block.indent = 0
        >>> lilypond_file.paper_block.top_margin = 15
        >>> lilypond_file.paper_block.left_margin = 15

    ..  doctest::

        >>> print format(lilypond_file) # doctest: +SKIP
        \header {
            composer = \markup { Josquin }
        }

        \layout {
            indent = #0
        }

        \paper {
            left-margin = #15
            top-margin = #15
        }

        \new Score <<
            \new Staff {
                c'8
                d'8
                e'8
                f'8
            }
        >>

    Equip LilyPond file with header, layout and paper blocks.

    Returns LilyPond file.
    '''
    from abjad.tools import lilypondfiletools
    if isinstance(music, lilypondfiletools.LilyPondFile):
        return music
    lilypond_file = lilypondfiletools.LilyPondFile()
    header_block = lilypondfiletools.HeaderBlock()
    layout_block = lilypondfiletools.LayoutBlock()
    paper_block = lilypondfiletools.PaperBlock()
    score_block = lilypondfiletools.ScoreBlock()
    lilypond_file.extend([
        header_block,
        layout_block,
        paper_block,
        score_block,
        ])
    lilypond_file.header_block = header_block
    lilypond_file.layout_block = layout_block
    lilypond_file.paper_block = paper_block
    lilypond_file.score_block = score_block
    if music is not None:
        score_block.append(music)
    return lilypond_file



# -*- encoding: utf-8 -*-
from abjad.tools import mathtools
import copy


def partition_sequence_by_backgrounded_weights(sequence, weights):
    r'''Partition `sequence` by backgrounded `weights`:

    ::

        >>> sequencetools.partition_sequence_by_backgrounded_weights(
        ...     [-5, -15, -10], [20, 10])
        [[-5, -15], [-10]]

    Further examples:

    ::

        >>> sequencetools.partition_sequence_by_backgrounded_weights(
        ...     [-5, -15, -10], [5, 5, 5, 5, 5, 5])
        [[-5], [-15], [], [], [-10], []]

    ::

        >>> sequencetools.partition_sequence_by_backgrounded_weights(
        ...     [-5, -15, -10], [1, 29])
        [[-5], [-15, -10]]

    ::

        >>> sequencetools.partition_sequence_by_backgrounded_weights(
        ...     [-5, -15, -10], [2, 28])
        [[-5], [-15, -10]]

    ::

        >>> sequencetools.partition_sequence_by_backgrounded_weights(
        ...     [-5, -15, -10], [1, 1, 1, 1, 1, 25])
        [[-5], [], [], [], [], [-15, -10]]

    The term `backgrounded` is a short-hand concocted specifically
    for this function; rely on the formal definition to understand
    the function actually does.

    Input constraint: the weight of `sequence` must equal the weight
    of `weights` exactly.

    The signs of the elements in `sequence` are ignored.

    Formal definition: partition `sequence` into `parts` such that
    (1.) the length of `parts` equals the length of `weights`;
    (2.) the elements in `sequence` appear in order in `parts`; and
    (3.) some final condition that is difficult to formalize.

    Notionally what's going on here is that the elements of `weights`
    are acting as a list of successive time intervals into which the
    elements of `sequence` are being fit in accordance with the start
    offset of each `sequence` element.

    The function models the grouping together of successive timespans
    according to which of an underlying sequence of time intervals
    it is in which each time span begins.

    Note that, for any input to this function, the flattened output
    of this function always equals `sequence` exactly.

    Note too that while `partition` is being used here in the sense of
    the other partitioning functions in the API, the distinguishing feature
    is this funciton is its ability to produce empty lists as output.

    Returns list of `sequence` objects.
    '''
    from abjad.tools import sequencetools

    assert all(0 < x for x in weights)
    assert mathtools.weight(sequence) == mathtools.weight(weights)

    start_offsets = \
        mathtools.cumulative_sums([abs(x) for x in sequence])[:-1]
    token = zip(start_offsets, sequence)

    result = []
    for interval_start, interval_stop in \
        mathtools.cumulative_sums_pairwise(weights):
        part = []
        for pair in token[:]:
            if interval_start <= pair[0] < interval_stop:
                part.append(pair[1])
                token.remove(pair)
        result.append(part)

    return result



# -*- encoding: utf-8 -*-
from abjad import *



def test_sequencetools_flatten_sequence_at_indices_01():
    r'''Works with positive indices.
    '''

    sequence_1 = [0, 1, [2, 3, 4], [5, 6, 7]]
    sequence_2 = sequencetools.flatten_sequence_at_indices(sequence_1, [2])

    assert sequence_2 == [0, 1, 2, 3, 4, [5, 6, 7]]


def test_sequencetools_flatten_sequence_at_indices_02():
    r'''Works with negative indices.
    '''

    sequence_1 = [0, 1, [2, 3, 4], [5, 6, 7]]
    works = sequencetools.flatten_sequence_at_indices(sequence_1, [-1])

    assert works == [0, 1, [2, 3, 4], 5, 6, 7]


def test_sequencetools_flatten_sequence_at_indices_03():
    r'''Boundary cases.
    '''

    sequence_1 = [0, 1, [2, 3, 4], [5, 6, 7]]

    sequence_2 = sequencetools.flatten_sequence_at_indices(sequence_1, [])
    assert sequence_2 == [0, 1, [2, 3, 4], [5, 6, 7]]

    sequence_2 = sequencetools.flatten_sequence_at_indices(sequence_1, [99])
    assert sequence_2 == [0, 1, [2, 3, 4], [5, 6, 7]]

    sequence_2 = sequencetools.flatten_sequence_at_indices(sequence_1, [-99])
    assert sequence_2 == [0, 1, [2, 3, 4], [5, 6, 7]]



# -*- encoding: utf-8 -*-
from abjad.tools import mathtools


def split_sequence_by_weights(sequence, weights, cyclic=False, overhang=False):
    '''Split sequence by weights.

    ..  container:: example

        **Example 1.** Split sequence cyclically by weights with overhang:

        ::

            >>> sequencetools.split_sequence_by_weights(
            ...     (10, -10, 10, -10), 
            ...     (3, 15, 3), 
            ...     cyclic=True, 
            ...     overhang=True,
            ...     )
            [(3,), (7, -8), (-2, 1), (3,), (6, -9), (-1,)]

    ..  container:: example
    
        **Example 2.** Split sequence cyclically by weights without overhang:

        ::

            >>> sequencetools.split_sequence_by_weights(
            ...     (10, -10, 10, -10), 
            ...     (3, 15, 3), 
            ...     cyclic=True, 
            ...     overhang=False,
            ...     )
            [(3,), (7, -8), (-2, 1), (3,), (6, -9)]

    ..  container:: example

        **Example 3.** Split sequence once by weights with overhang:

        ::

            >>> sequencetools.split_sequence_by_weights(
            ...     (10, -10, 10, -10), 
            ...     (3, 15, 3), 
            ...     cyclic=False, 
            ...     overhang=True,
            ...     )
            [(3,), (7, -8), (-2, 1), (9, -10)]

    ..  container:: example

        **Example 4.** Split sequence once by weights without overhang:

        ::

            >>> sequencetools.split_sequence_by_weights(
            ...     (10, -10, 10, -10), 
            ...     (3, 15, 3), 
            ...     cyclic=False, 
            ...     overhang=False,
            ...     )
            [(3,), (7, -8), (-2, 1)]

    Returns list of sequence types.
    '''
    from abjad.tools import sequencetools

    result = []
    current_index = 0
    current_piece = []

    if cyclic:
        weights = sequencetools.repeat_sequence_to_weight_at_most(weights, mathtools.weight(sequence))

    for weight in weights:
        current_piece_weight = mathtools.weight(current_piece)
        while current_piece_weight < weight:
            current_piece.append(sequence[current_index])
            current_index += 1
            current_piece_weight = mathtools.weight(current_piece)
        if current_piece_weight == weight:
            current_piece = type(sequence)(current_piece)
            result.append(current_piece)
            current_piece = []
        elif weight < current_piece_weight:
            overage = current_piece_weight - weight
            current_last_element = current_piece.pop(-1)
            needed = abs(current_last_element) - overage
            needed *= mathtools.sign(current_last_element)
            current_piece.append(needed)
            current_piece = type(sequence)(current_piece)
            result.append(current_piece)
            overage *= mathtools.sign(current_last_element)
            current_piece = [overage]

    if overhang:
        last_piece = current_piece
        last_piece.extend(sequence[current_index:])
        if last_piece:
            last_piece = type(sequence)(last_piece)
            result.append(last_piece)

    return result



# -*- encoding: utf-8 -*-
from abjad import *


def test_sequencetools_negate_sequence_elements_at_indices_01():

    sequence_1 = [1, 2, 3, 4, 5, 6, 7, 8, 9, 10]
    sequence_2 = sequencetools.negate_sequence_elements_at_indices(sequence_1, [0, 1, 2])

    assert sequence_2 == [-1, -2, -3, 4, 5, 6, 7, 8, 9, 10]



# -*- encoding: utf-8 -*-


def reverse_sequence(sequence):
    '''Reverse `sequence`:

    ::

        >>> sequencetools.reverse_sequence((1, 2, 3, 4, 5))
        (5, 4, 3, 2, 1)

    Returns new `sequence` object.
    '''

    return type(sequence)(reversed(sequence))



# -*- encoding: utf-8 -*-
from abjad import *


def test_pitcharraytools_PitchArrayInventory_to_score_01():

    array_1 = pitcharraytools.PitchArray([
        [1, (2, 1), ([-2, -1.5], 2)],
        [(7, 2), (6, 1), 1],
        ])

    array_2 = pitcharraytools.PitchArray([
        [1, 1, 1],
        [1, 1, 1]])

    arrays = [array_1, array_2]
    inventory = pitcharraytools.PitchArrayInventory(arrays)
    score = inventory.to_score()

    r'''
    \new Score <<
        \new StaffGroup <<
            \new Staff {
                {
                    \time 4/8
                    r8
                    d'8
                    <bf bqf>4
                }
                {
                    \time 3/8
                    r8
                    r8
                    r8
                }
            }
            \new Staff {
                {
                    \time 4/8
                    g'4
                    fs'8
                    r8
                }
                {
                    \time 3/8
                    r8
                    r8
                    r8
                }
            }
        >>
    >>
    '''

    assert inspect(score).is_well_formed()
    assert systemtools.TestManager.compare(
        score,
        r'''
        \new Score <<
            \new StaffGroup <<
                \new Staff {
                    {
                        \time 4/8
                        r8
                        d'8
                        <bf bqf>4
                    }
                    {
                        \time 3/8
                        r8
                        r8
                        r8
                    }
                }
                \new Staff {
                    {
                        \time 4/8
                        g'4
                        fs'8
                        r8
                    }
                    {
                        \time 3/8
                        r8
                        r8
                        r8
                    }
                }
            >>
        >>
        '''
        )



# -*- encoding: utf-8 -*-
from abjad.tools import stringtools
from abjad.tools.lilypondnametools.LilyPondNameManager \
    import LilyPondNameManager


class LilyPondSettingNameManager(LilyPondNameManager):
    '''LilyPond setting name manager.
    '''

    ### CLASS VARIABLES ###

    skeleton_string_prefix = 'set__'

    ### SPECIAL METHODS ###

    def __getattr__(self, name):
        r'''Gets setting `name` from LilyPond setting name manager.

        Returns string.
        '''
        from abjad import ly
        from abjad.tools import lilypondnametools
        camel_name = stringtools.snake_case_to_upper_camel_case(name)
        if name.startswith('_'):
            try:
                return vars(self)[name]
            except KeyError:
                message = '{!r} object has no attribute: {!r}.'
                message = message.format(type(self).__name__, name)
                raise AttributeError(message)
        elif camel_name in ly.contexts:
            try:
                return vars(self)['_' + name]
            except KeyError:
                context = lilypondnametools.LilyPondNameManager()
                vars(self)['_' + name] = context
                return context
        else:
            try:
                return vars(self)[name]
            except KeyError:
                message = '{!r} object has no attribute: {!r}.'
                message = message.format(type(self).__name__, name)
                raise AttributeError(message)

    ### PRIVATE METHODS ###

    def _get_attribute_tuples(self):
        from abjad.tools import lilypondnametools
        result = []
        for name, value in vars(self).iteritems():
            if type(value) is lilypondnametools.LilyPondNameManager:
                prefixed_context_name = name
                context_name = prefixed_context_name.strip('_')
                context_proxy = value
                attribute_pairs = context_proxy._get_attribute_pairs()
                for attribute_name, attribute_value in attribute_pairs:
                    triple = (context_name, attribute_name, attribute_value)
                    result.append(triple)
            else:
                attribute_name, attribute_value = name, value
                result.append((attribute_name, attribute_value))
        return result

    def _get_skeleton_strings(self):
        result = []
        for attribute_tuple in self._get_attribute_tuples():
            if len(attribute_tuple) == 2:
                attribute_name, attribute_value = attribute_tuple
                string = '{}={}'.format(attribute_name, repr(attribute_value))
                result.append(string)
            elif len(attribute_tuple) == 3:
                context_name, attribute_name, attribute_value = attribute_tuple
                key = '__'.join((context_name, attribute_name))
                string = '{}={}'.format(key, repr(attribute_value))
                result.append(string)
            else:
                message = 'attribute tuple must have length 2 or 3.'
                raise ValueError(message)
        result = ['set__' + x for x in result]
        return result



# -*- encoding: utf-8 -*-


def persist(expr):
    r'''Persists `expr`.

    ..  container:: example

        ::

            >>> staff = Staff("c'4 e'4 d'4 f'4")
            >>> show(staff) # doctest: +SKIP

        ::

            >>> persist(staff)
            PersistenceAgent(client={c'4, e'4, d'4, f'4})

    Returns score mutation agent.
    '''
    from abjad.tools import agenttools
    return agenttools.PersistenceAgent(expr)



# -*- encoding: utf-8 -*-
from abjad import *
from abjad.tools import sequencetools


def test_scoretools_make_repeated_notes_from_time_signatures_01():
    r'''Make repeated notes from list of integer pairs.
    '''

    notes = scoretools.make_repeated_notes_from_time_signatures([(2, 8), (3, 32)], pitch = "d''")
    assert len(notes) == 2

    notes = sequencetools.flatten_sequence(notes)
    staff = Staff(notes)

    r'''
    \new Staff {
        d''8
        d''8
        d''32
        d''32
        d''32
    }
    '''

    assert systemtools.TestManager.compare(
        staff,
        r'''
        \new Staff {
            d''8
            d''8
            d''32
            d''32
            d''32
        }
        '''
        )
    assert inspect(staff).is_well_formed()



# -*- encoding: utf-8 -*-
from abjad.tools import scoretools
from abjad.tools import scoretools
from abjad.tools.spannertools.Spanner import Spanner


class Glissando(Spanner):
    r'''A glissando spanner.

    ::

        >>> staff = Staff("c'8 d'8 e'8 f'8")
        >>> glissando = spannertools.Glissando()
        >>> attach(glissando, staff[:])
        >>> show(staff) # doctest: +SKIP

    ..  doctest::

        >>> print format(staff)
        \new Staff {
            c'8 \glissando
            d'8 \glissando
            e'8 \glissando
            f'8
        }

    Can avoid rests:

    ::

        >>> staff = Staff("c'16 r r g' r8 c'8")
        >>> glissando = spannertools.Glissando(avoid_rests=True)
        >>> attach(glissando, staff[:])
        >>> beam = Beam()
        >>> attach(beam, staff[:])
        >>> show(staff) # doctest: +SKIP

    ..  doctest::

        >>> print format(staff)
        \new Staff {
            c'16 [ \glissando
            \once \override NoteColumn #'glissando-skip = ##t
            \once \override Rest #'transparent = ##t
            r16
            \once \override NoteColumn #'glissando-skip = ##t
            \once \override Rest #'transparent = ##t
            r16
            g'16 \glissando
            \once \override NoteColumn #'glissando-skip = ##t
            \once \override Rest #'transparent = ##t
            r8
            c'8 ]
        }

    Formats nonlast leaves in spanner with LilyPond glissando command.
    '''

    ### INITIALIZER ###

    def __init__(
        self,
        components=None,
        overrides=None,
        avoid_rests=False,
        ):
        Spanner.__init__(
            self,
            components,
            overrides=overrides,
            )
        self._avoid_rests = bool(avoid_rests)

    ### PRIVATE METHODS ###

    def _copy_keyword_args(self, new):
        pass

    def _format_before_leaf(self, leaf):
        result = []
        if self.avoid_rests:
            if not isinstance(leaf, (scoretools.Chord, scoretools.Note)):
                result.append(
                    r"\once \override NoteColumn #'glissando-skip = ##t")
                result.append(r"\once \override Rest #'transparent = ##t")
        return result

    def _format_right_of_leaf(self, leaf):
        r'''Spanner contribution to right of leaf.
        '''
        result = []
        if not self._is_my_last_leaf(leaf) and \
            isinstance(leaf, (scoretools.Chord, scoretools.Note)):
            result.append(r'\glissando')
        return result

    ### PUBLIC PROPERTIES ###

    @property
    def avoid_rests(self):
        r'''Gets and sets rest avoidance.

        ..  container:: example

            Gets property:

            ::

                >>> glissando.avoid_rests
                True

        ..  container:: example

            Sets property:

            ::

                >>> glissando.avoid_rests = False
                >>> glissando.avoid_rests
                False

        Returns boolean.
        '''
        return self._avoid_rests

    @avoid_rests.setter
    def avoid_rests(self, expr):
        self._avoid_rests = bool(expr)



# -*- encoding: utf-8 -*-
from abjad import *


def test_markuptools_MarkupCommand___format___01():

    a = markuptools.MarkupCommand('draw-circle', 1, 0.1, False)
    b = markuptools.MarkupCommand('line', ['one', 'two', 'three'])
    c = markuptools.MarkupCommand('rounded-box', b)
    d = markuptools.MarkupCommand('combine', a, c)
    e = markuptools.MarkupCommand('rotate', 45, d)
    f = markuptools.MarkupCommand('triangle', False)
    g = markuptools.MarkupCommand('concat', [e, f])

    assert systemtools.TestManager.compare(
        format(g, 'lilypond'),
        r'''
        \concat
            {
                \rotate
                    #45
                    \combine
                        \draw-circle
                            #1
                            #0.1
                            ##f
                        \rounded-box
                            \line
                                {
                                    one
                                    two
                                    three
                                }
                \triangle
                    ##f
            }
        ''',
        )



# -*- encoding: utf-8 -*-
from abjad import *
from abjad.tools import systemtools


def test_stringtools_snake_case_to_lower_camel_case_01():

    assert stringtools.snake_case_to_lower_camel_case('foo_bar_blah') == 'fooBarBlah'



# -*- encoding: utf-8 -*-
import os
import pytest
import shutil
from abjad.tools import *


@pytest.skip('Test runs slowly.')
def test_abjadbooktools_AbjadBookProcessor___call___01():

    if __name__ == '__main__':
        directory = os.path.curdir
    else:
        directory = os.path.dirname(__file__)

    file_path = os.path.join(directory, 'text.rst.raw')

    with open(file_path, 'r') as f:
        lines = f.read().split('\n')

    book = abjadbooktools.AbjadBookProcessor(
        directory, lines, abjadbooktools.ReSTOutputFormat())

    result = book(directory)

    assert systemtools.TestManager.compare(
        result,
        r'''
        This is **paragraph 1**.
        Now comes some Abjad code

        ::

        >>> voice = Voice("c'8 d'8 e'8")
        >>> Beam(voice)
        Beam({c'8, d'8, e'8})
        >>> beam = _
        >>> show(voice)

        .. image:: images/image-1.png

        ::

        >>> len(beam)
        1
        >>> show(voice)

        .. image:: images/image-2.png


        Here is **paragraph 2**, and more Abjad code.
        Notice that in the second block of abjad code I can reference objects and variables created in previous blocks:

        ::

        >>> spannertools.TrillSpanner(voice[4:])
        TrillSpanner()
        >>> print format(voice)
        \new Voice {
            c'8 [
            d'8
            e'8 ]
        }
        >>> show(voice)

        .. image:: images/image-3.png


        Here is **paragraph 3**, and now a function definition.
        Note that this function definition can be used later.
        The **strip_prompt=true** option tells abjad-book to print the code block as though it wasn't passed to the interpreter.

        ::

        def apply_articulations(components):
            for i, component in enumerate(components):
                if i % 2 == 0:
                    Articulation('.')(component)
                else:
                    Articulation('^')(component)


        Here is **paragraph** 4, where we use the previous function to change the Voice we previously instantiated.

        ::

        >>> apply_articulations(voice)
        >>> show(voice)

        .. image:: images/image-4.png


        And a final paragraph.

        '''
        )

    assert os.path.exists(os.path.join(directory, 'images'))
    shutil.rmtree(os.path.join(directory, 'images'))

    return result



# -*- encoding: utf-8 -*-
from abjad import *


def test_sequencetools_increase_sequence_elements_at_indices_by_addenda_01():
    r'''Increase elements of list sequence_1 by the elements of addenda
    at indices in sequence_1.
    '''

    sequence_1 = [1, 1, 2, 3, 5, 5, 1, 2, 5, 5, 6]
    sequence_2 = sequencetools.increase_sequence_elements_at_indices_by_addenda(sequence_1, [0.5, 0.5], [0, 4, 8])
    assert sequence_2 == [1.5, 1.5, 2, 3, 5.5, 5.5, 1, 2, 5.5, 5.5, 6]



# -*- encoding: utf-8 -*-
from abjad import *
from abjad.tools import *


def test_rhythmmakertools_TupletMonadRhythmMaker___call___01():

    maker = rhythmmakertools.TupletMonadRhythmMaker()

    divisions = [(1, 5), (1, 4), (1, 6), (7, 9)]
    tuplet_lists = maker(divisions)
    tuplets = sequencetools.flatten_sequence(tuplet_lists)
    staff = Staff(tuplets)

    r'''
    \new Staff {
        \times 4/5 {
            c'4
        }
        {
            c'4
        }
        \times 2/3 {
            c'4
        }
        \times 8/9 {
            c'2..
        }
    }
    '''

    assert systemtools.TestManager.compare(
        staff,
        r'''
        \new Staff {
            \times 4/5 {
                c'4
            }
            {
                c'4
            }
            \times 2/3 {
                c'4
            }
            \times 8/9 {
                c'2..
            }
        }
        '''
        )



# -*- encoding: utf-8 -*-


def weight(sequence):
    r'''Sum of the absolute value of the elements in `sequence`:

    ::

        >>> mathtools.weight([-1, -2, 3, 4, 5])
        15

    Returns nonnegative integer.
    '''

    return sum([abs(element) for element in sequence])



# -*- encoding: utf-8 -*-
from abjad import *


def test_markuptools_MarkupCommand_command_01():
    mc = markuptools.MarkupCommand('draw-circle', 1, 0.1, False)
    assert mc.command == 'draw-circle'



# -*- encoding: utf-8 -*-


def remove_subsequence_of_weight_at_index(sequence, weight, index):
    '''Remove subsequence of `weight` at `index`:

    ::

        >>> sequence = (1, 1, 2, 3, 5, 5, 1, 2, 5, 5, 6)

    ::

        >>> sequencetools.remove_subsequence_of_weight_at_index(sequence, 13, 4)
        (1, 1, 2, 3, 5, 5, 6)

    Returns newly constructed `sequence` object.
    '''

    result = list(sequence[:index])
    total = 0
    for element in sequence[index:]:
        if weight <= total:
            result.append(element)
        elif weight < total + element:
            result.append(total + element - weight)
        total += element
    return type(sequence)(result)



# -*- encoding: utf-8 -*-
from abjad import *
from abjad.tools.mathtools import NonreducedFraction


def test_mathtools_NonreducedFraction___add___01():

    result = NonreducedFraction(1, 4) + NonreducedFraction(2, 8)
    assert result.pair == (4, 8)

    result = NonreducedFraction(2, 8) + NonreducedFraction(1, 4)
    assert result.pair == (4, 8)


def test_mathtools_NonreducedFraction___add___02():

    result = NonreducedFraction(3, 3) + 1
    assert result.pair == (6, 3)

    result = 1 + NonreducedFraction(3, 3)
    assert result.pair == (6, 3)



# -*- encoding: utf-8 -*-
from abjad.tools.abctools import AbjadObject


class LilyPondFormatBundle(AbjadObject):
    r'''LilyPond format bundle.

    Transient class created to hold the collection of all
    format contributions generated on behalf of a single component.
    '''

    ### CLASS VARIABLES ###

    __slots__ =  (
        '_before',
        '_after',
        '_opening',
        '_closing',
        '_right',
        '_context_settings',
        '_grob_overrides',
        '_grob_reverts',
        )

    ### INNER CLASS DEFINITION ###

    class SlotContributions(object):
        r'''Slot contributions.
        '''
        
        __slots__ = (
            '_articulations',
            '_commands',
            '_comments',
            '_indicators',
            '_markup',
            '_spanners',
            '_stem_tremolos',
            )

        def __init__(self):
            self._articulations = []
            self._commands = []
            self._comments = []
            self._indicators = []
            self._markup = []
            self._spanners = []
            self._stem_tremolos = []

        @property
        def articulations(self):
            return self._articulations

        @property
        def comments(self):
            return self._comments

        @property
        def commands(self):
            return self._commands

        @property
        def indicators(self):
            return self._indicators

        @property
        def markup(self):
            return self._markup

        @property
        def spanners(self):
            return self._spanners

        @property
        def stem_tremolos(self):
            return self._stem_tremolos

        def alphabetize(self):
            self._indicators.sort()

        def get(self, identifier):
            return getattr(self, identifier) 

        def make_immutable(self):
            self._articulations = tuple(sorted(self.articulations))
            self._commands = tuple(self.commands)
            self._comments = tuple(self.comments)
            self._indicators = tuple(self.indicators)
            self._markup = tuple(self.markup)
            self._spanners = tuple(self.spanners)
            self._stem_tremolos = tuple(self.stem_tremolos)
    
    ### INITIALIZER ###

    def __init__(self):
        self._before = self.SlotContributions()
        self._after = self.SlotContributions()
        self._opening = self.SlotContributions()
        self._closing = self.SlotContributions()
        self._right = self.SlotContributions()
        self._context_settings = []
        self._grob_overrides = []
        self._grob_reverts = []

    ### PUBLIC PROPERTIES ###

    @property
    def after(self):
        r'''After slot contributions.

        Returns slot contributions object.
        '''
        return self._after

    @property
    def before(self):
        r'''Before slot contributions.

        Returns slot contributions object.
        '''
        return self._before

    @property
    def closing(self):
        r'''Closing slot contributions.

        Returns slot contributions object.
        '''
        return self._closing

    @property
    def context_settings(self):
        r'''Context setting format contributions.

        Returns list.
        '''
        return self._context_settings

    @property
    def grob_overrides(self):
        r'''Grob override format contributions.

        Returns list.
        '''
        return self._grob_overrides

    @property
    def grob_reverts(self):
        r'''Grob revert format contributions.

        Returns list.
        '''
        return self._grob_reverts

    @property
    def opening(self):
        r'''Opening slot contributions.

        Returns slot contributions object.
        '''
        return self._opening

    @property
    def right(self):
        r'''Right slot contributions.

        Returns slot contributions object.
        '''
        return self._right

    ### PUBLIC METHODS ###

    def alphabetize(self):
        r'''Alphabetize format contributions in each slot.

        Returns none.
        '''
        self.before.alphabetize()
        self.after.alphabetize()
        self.opening.alphabetize()
        self.closing.alphabetize()
        self.right.alphabetize()
        self._context_settings.sort()
        self._grob_overrides.sort()
        self._grob_reverts.sort()
        
    def get(self, identifier):
        r'''Get `identifier`.

        Returns format contributions object or list.
        '''
        return getattr(self, identifier)

    def make_immutable(self):
        r'''Make each slot immutable.
        
        Returns none.
        '''
        self.before.make_immutable()
        self.after.make_immutable()
        self.opening.make_immutable()
        self.closing.make_immutable()
        self.right.make_immutable()
        self._context_settings = tuple(self.context_settings)
        self._grob_overrides = tuple(self.grob_overrides)
        self._grob_reverts = tuple(self.grob_reverts)



# -*- encoding: utf-8 -*-
#! /usr/bin/env python

if __name__ == '__main__':
    from abjad.demos import desordre
    from abjad.tools import systemtools
    lilypond_file = desordre.make_desordre_lilypond_file()
    topleveltools.show(lilypond_file)



# -*- encoding: utf-8 -*-
import copy
from abjad import *


def test_scoretools_Tuplet___copy___01():

    tuplet_1 = Tuplet((2, 3), "c'8 d'8 e'8")
    override(tuplet_1).note_head.color = 'red'

    assert systemtools.TestManager.compare(
        tuplet_1,
        r'''
        \override NoteHead #'color = #red
        \times 2/3 {
            c'8
            d'8
            e'8
        }
        \revert NoteHead #'color
        '''
        )

    tuplet_2 = copy.copy(tuplet_1)


    assert systemtools.TestManager.compare(
        tuplet_2,
        r'''
        \override NoteHead #'color = #red
        \times 2/3 {
        }
        \revert NoteHead #'color
        '''
        )

    assert inspect(tuplet_2).is_well_formed()



# -*- encoding: utf-8 -*-


def increase_sequence_elements_cyclically_by_addenda(sequence, addenda, shield=True):
    '''Increase `sequence` cyclically by `addenda`:

    ::

        >>> sequencetools.increase_sequence_elements_cyclically_by_addenda(
        ...     range(10), [10, -10], shield=False)
        [10, -9, 12, -7, 14, -5, 16, -3, 18, -1]

    Increase `sequence` cyclically by addenda and map nonpositive values to ``1``:

    ::

        >>> sequencetools.increase_sequence_elements_cyclically_by_addenda(
        ...         range(10), [10, -10], shield=True)
        [10, 1, 12, 1, 14, 1, 16, 1, 18, 1]

    Returns list.
    '''

    if not isinstance(sequence, (list, tuple)):
        raise TypeError

    result = []

    for i, element in enumerate(sequence):
        new = element + addenda[i % len(addenda)]
        if shield and new <= 0:
            new = 1
        result.append(new)

    return result



# -*- encoding: utf-8 -*-
from abjad import *


def test_mathtools_NonreducedFraction_with_denominator_01():
    assert mathtools.NonreducedFraction((0, 6)).with_denominator(12) == mathtools.NonreducedFraction(0, 12)
    assert mathtools.NonreducedFraction((1, 6)).with_denominator(12) == mathtools.NonreducedFraction(2, 12)
    assert mathtools.NonreducedFraction((2, 6)).with_denominator(12) == mathtools.NonreducedFraction(4, 12)
    assert mathtools.NonreducedFraction((3, 6)).with_denominator(12) == mathtools.NonreducedFraction(6, 12)
    assert mathtools.NonreducedFraction((4, 6)).with_denominator(12) == mathtools.NonreducedFraction(8, 12)
    assert mathtools.NonreducedFraction((5, 6)).with_denominator(12) == mathtools.NonreducedFraction(10, 12)
    assert mathtools.NonreducedFraction((6, 6)).with_denominator(12) == mathtools.NonreducedFraction(12, 12)
    assert mathtools.NonreducedFraction((7, 6)).with_denominator(12) == mathtools.NonreducedFraction(14, 12)
    assert mathtools.NonreducedFraction((8, 6)).with_denominator(12) == mathtools.NonreducedFraction(16, 12)
    assert mathtools.NonreducedFraction((9, 6)).with_denominator(12) == mathtools.NonreducedFraction(18, 12)
    assert mathtools.NonreducedFraction((10, 6)).with_denominator(12) == mathtools.NonreducedFraction(20, 12)
    assert mathtools.NonreducedFraction((11, 6)).with_denominator(12) == mathtools.NonreducedFraction(22, 12)


def test_mathtools_NonreducedFraction_with_denominator_02():
    assert mathtools.NonreducedFraction((0, 12)).with_denominator(6) == mathtools.NonreducedFraction(0, 6)
    assert mathtools.NonreducedFraction((1, 12)).with_denominator(6) == mathtools.NonreducedFraction(1, 12)
    assert mathtools.NonreducedFraction((2, 12)).with_denominator(6) == mathtools.NonreducedFraction(1, 6)
    assert mathtools.NonreducedFraction((3, 12)).with_denominator(6) == mathtools.NonreducedFraction(3, 12)
    assert mathtools.NonreducedFraction((4, 12)).with_denominator(6) == mathtools.NonreducedFraction(2, 6)
    assert mathtools.NonreducedFraction((5, 12)).with_denominator(6) == mathtools.NonreducedFraction(5, 12)
    assert mathtools.NonreducedFraction((6, 12)).with_denominator(6) == mathtools.NonreducedFraction(3, 6)
    assert mathtools.NonreducedFraction((7, 12)).with_denominator(6) == mathtools.NonreducedFraction(7, 12)
    assert mathtools.NonreducedFraction((8, 12)).with_denominator(6) == mathtools.NonreducedFraction(4, 6)
    assert mathtools.NonreducedFraction((9, 12)).with_denominator(6) == mathtools.NonreducedFraction(9, 12)
    assert mathtools.NonreducedFraction((10, 12)).with_denominator(6) == mathtools.NonreducedFraction(5, 6)
    assert mathtools.NonreducedFraction((11, 12)).with_denominator(6) == mathtools.NonreducedFraction(11, 12)


def test_mathtools_NonreducedFraction_with_denominator_03():
    assert mathtools.NonreducedFraction((0, 12)).with_denominator(8) == mathtools.NonreducedFraction(0, 8)
    assert mathtools.NonreducedFraction((1, 12)).with_denominator(8) == mathtools.NonreducedFraction(1, 12)
    assert mathtools.NonreducedFraction((2, 12)).with_denominator(8) == mathtools.NonreducedFraction(2, 12)
    assert mathtools.NonreducedFraction((3, 12)).with_denominator(8) == mathtools.NonreducedFraction(2, 8)
    assert mathtools.NonreducedFraction((4, 12)).with_denominator(8) == mathtools.NonreducedFraction(4, 12)
    assert mathtools.NonreducedFraction((5, 12)).with_denominator(8) == mathtools.NonreducedFraction(5, 12)
    assert mathtools.NonreducedFraction((6, 12)).with_denominator(8) == mathtools.NonreducedFraction(4, 8)
    assert mathtools.NonreducedFraction((7, 12)).with_denominator(8) == mathtools.NonreducedFraction(7, 12)
    assert mathtools.NonreducedFraction((8, 12)).with_denominator(8) == mathtools.NonreducedFraction(8, 12)
    assert mathtools.NonreducedFraction((9, 12)).with_denominator(8) == mathtools.NonreducedFraction(6, 8)
    assert mathtools.NonreducedFraction((10, 12)).with_denominator(8) == mathtools.NonreducedFraction(10, 12)
    assert mathtools.NonreducedFraction((11, 12)).with_denominator(8) == mathtools.NonreducedFraction(11, 12)



# -*- encoding: utf-8 -*-
import re


underscore_delimited_lowercase_regex_body = '(([a-z,0-9]+[_]+)*[a-z,0-9]+)?'
underscore_delimited_lowercase_regex = re.compile('^%s$' % underscore_delimited_lowercase_regex_body)

def is_snake_case_string(expr):
    r'''True when `expr` is a string and is underscore delimited lowercase:

    ::

        >>> stringtools.is_snake_case_string('foo_bar')
        True

    Otherwise false:

    ::

        >>> stringtools.is_snake_case_string('foo bar')
        False

    Returns boolean.
    '''

    if not isinstance(expr, str):
        return False

    return bool(underscore_delimited_lowercase_regex.match(expr))



# -*- encoding: utf-8 -*-
from abjad.tools.datastructuretools.TypedList import TypedList
from abjad.tools.pitchtools.OctaveTranspositionMappingComponent \
    import OctaveTranspositionMappingComponent


class OctaveTranspositionMapping(TypedList):
    '''An octave transposition mapping.

    ::

        >>> mapping = pitchtools.OctaveTranspositionMapping(
        ...     [('[A0, C4)', 15), ('[C4, C8)', 27)])

    ::

        >>> mapping
        OctaveTranspositionMapping([('[A0, C4)', 15), ('[C4, C8)', 27)])

    Octave transposition mappings model
    ``pitchtools.transpose_pitch_number_by_octave_transposition_mapping``
    input.

    Octave transposition mappings implement the list interface and
    are mutable.
    '''

    ### SPECIAL METHODS ###

    def __call__(self, pitches):
        r'''Call octave transposition mapping on `pitches`.

            >>> mapping([-24, -22, -23, -21])
            [24, 26, 25, 15]

        ::

            >>> mapping([0, 2, 1, 3])
            [36, 38, 37, 27]

        Returns list.
        '''
        transposed_pitches = [self._transpose_pitch(x) for x in pitches]
        return transposed_pitches

    def __format__(self, format_specification=''):
        r'''Formats octave transposition mapping.

        Set `format_specification` to `''` or `'storage'`.
        Interprets `''` equal to `'storage'`.

        ::

            >>> print format(mapping)
            pitchtools.OctaveTranspositionMapping(
                [
                    pitchtools.OctaveTranspositionMappingComponent(
                        pitchtools.PitchRange(
                            '[A0, C4)'
                            ),
                        pitchtools.NumberedPitch(15)
                        ),
                    pitchtools.OctaveTranspositionMappingComponent(
                        pitchtools.PitchRange(
                            '[C4, C8)'
                            ),
                        pitchtools.NumberedPitch(27)
                        ),
                    ]
                )

        Returns string.
        '''
        superclass = super(OctaveTranspositionMapping, self)
        return superclass.__format__(format_specification=format_specification)

    ### PRIVATE METHODS ###

    def _transpose_pitch(self, pitch):
        from abjad.tools import pitchtools
        pitch = pitchtools.NamedPitch(pitch)
        target_pitch_class_number = pitch.pitch_class_number
        for component in self:
            if pitch in component.source_pitch_range:
                target_octave = range(
                    component.target_octave_start_pitch.pitch_number,
                    component.target_octave_start_pitch.pitch_number
                    + 12)
                for candidate_pitch in target_octave:
                    if candidate_pitch % 12 == \
                        pitch.pitch_class_number:
                        return candidate_pitch
        return pitch

    ### PRIVATE PROPERTIES ###

    @property
    def _item_callable(self):
        return OctaveTranspositionMappingComponent

    @property
    def _one_line_menuing_summary(self):
        name = self.custom_identifier or 'mapping'
        contents = []
        for mapping_component in self:
            contents.append(mapping_component._one_line_menuing_summary)
        contents_string = ', '.join(contents)
        return '{}: {}'.format(name, contents_string)

    @property
    def _repr_specification(self):
        from abjad.tools import systemtools
        input_argument_tokens = []
        for mapping_component in self:
            token = (
                mapping_component.source_pitch_range.one_line_named_pitch_repr,
                mapping_component.target_octave_start_pitch.pitch_number
                )
            input_argument_tokens.append(token)
        keyword_argument_names = []
        if self.custom_identifier is not None:
            keyword_argument_names.append('custom_identifier')
        return systemtools.StorageFormatSpecification(
            self,
            is_indented=False,
            keyword_argument_names=keyword_argument_names,
            positional_argument_values=(
                input_argument_tokens,
                ),
            )



# -*- encoding: utf-8 -*-
from abjad.tools import durationtools
from abjad.tools import mathtools
from abjad.tools.abctools.AbjadObject import AbjadObject


# TODO: add more initializer examples.
# TODO: add example of `suppress` keyword.
# TODO: turn `suppress` into managed attribute.
class TimeSignature(AbjadObject):
    r'''A time signature.

    ::

        >>> staff = Staff("c'8 d'8 e'8 f'8")
        >>> time_signature = TimeSignature((4, 8))
        >>> attach(time_signature, staff[0])
        >>> show(staff) # doctest: +SKIP

    ..  doctest::

        >>> print format(staff)
        \new Staff {
            \time 4/8
            c'8
            d'8
            e'8
            f'8
        }

    Time signatures are scoped to the **staff** by default.

    Set the scope of time signatures to the **score** like this:

    ::

        >>> staff = Staff("c'8 d'8 e'8 f'8")
        >>> time_signature = TimeSignature((4, 8))
        >>> attach(time_signature, staff[0], scope=Score)
        >>> show(staff) # doctest: +SKIP

    '''

    ### CLASS VARIABLES ###

    _default_positional_input_arguments = (
        (4, 8),
        )

    _format_slot = 'opening'

    ### INITIALIZER ###

    def __init__(self, *args, **kwargs):
        from abjad.tools import scoretools
        self._default_scope = scoretools.Staff
        partial, suppress = None, None
        # initialize numerator and denominator from *args
        if len(args) == 1 and isinstance(args[0], type(self)):
            time_signature = args[0]
            numerator = time_signature.numerator
            denominator = time_signature.denominator
            partial = time_signature.partial
            suppress = time_signature.suppress
        elif len(args) == 1 and isinstance(args[0], durationtools.Duration):
            numerator, denominator = args[0].numerator, args[0].denominator
        elif len(args) == 1 and isinstance(args[0], tuple):
            numerator, denominator = args[0][0], args[0][1]
        elif len(args) == 1 and hasattr(args[0], 'numerator') and \
            hasattr(args[0], 'denominator'):
            numerator, denominator = args[0].numerator, args[0].denominator
        else:
            message = 'invalid time_signature initialization: {!r}.'
            message = message.format(args)
            raise TypeError(message)
        self._numerator = numerator
        self._denominator = denominator
        # initialize partial from **kwargs
        partial = partial or kwargs.get('partial', None)
        if not isinstance(partial, (type(None), durationtools.Duration)):
            raise TypeError
        self._partial = partial
        if partial is not None:
            self._partial_repr_string = ', partial=%r' % self._partial
        else:
            self._partial_repr_string = ''
        # initialize suppress from kwargs
        suppress = suppress or kwargs.get('suppress', None)
        if not isinstance(suppress, (bool, type(None))):
            raise TypeError
        self.suppress = suppress
        # initialize derived attributes
        self._multiplier = self.implied_prolation
        self._has_non_power_of_two_denominator = \
            not mathtools.is_nonnegative_integer_power_of_two(
            self.denominator)

    ### SPECIAL METHODS ###

    def __copy__(self, *args):
        r'''Copies time signature.

        Returns new time signature.
        '''
        return type(self)(
            (self.numerator, self.denominator),
            partial=self.partial, 
            )

    def __eq__(self, arg):
        r'''True when `arg` is a time signature with numerator and denominator
        equal to this time signature. Also true when `arg` is a tuple with
        first and second elements equal to numerator and denominator of this
        time signature. Otherwise false.

        Returns boolean.
        '''
        if isinstance(arg, type(self)):
            return self.numerator == arg.numerator and \
                self.denominator == arg.denominator
        elif isinstance(arg, tuple):
            return self.numerator == arg[0] and self.denominator == arg[1]
        else:
            return False

    def __format__(self, format_specification=''):
        r'''Formats time signature.

        ::

            >>> print format(TimeSignature((3, 8)))
            indicatortools.TimeSignature(
                (3, 8)
                )

        Returns string.
        '''
        from abjad.tools import systemtools
        if format_specification in ('', 'storage'):
            return systemtools.StorageFormatManager.get_storage_format(self)
        elif format_specification == 'lilypond':
            return self._lilypond_format
        return str(self)

    def __ge__(self, arg):
        r'''True when duration of time signature is greater than or equal to
        duration of `arg`. Otherwise false.

        Returns boolean.
        '''
        if isinstance(arg, type(self)):
            return self.duration >= arg.duration
        else:
            raise TypeError

    def __gt__(self, arg):
        r'''True when duration of time signature is greater than duration of 
        `arg`. Otherwise false.

        Returns boolean.
        '''
        if isinstance(arg, type(self)):
            return self.duration > arg.duration
        else:
            raise TypeError

    def __le__(self, arg):
        r'''True when duration of time signature is less than duration of
        `arg`. Otherwise false.

        Returns boolean.
        '''
        if isinstance(arg, type(self)):
            return self.duration <= arg.duration
        else:
            raise TypeError

    def __lt__(self, arg):
        r'''True when duration of time signature is less than duration of
        `arg`. Otherwise false.

        Returns booelan.
        '''
        if isinstance(arg, type(self)):
            return self.duration < arg.duration
        else:
            raise TypeError

    def __str__(self):
        r'''String representation of time signature.

        Returns string.
        '''
        return '{}/{}'.format(self.numerator, self.denominator)

    ### PRIVATE PROPERTIES ###

    @property
    def _contents_repr_string(self):
        return '{}/{}'.format(self.numerator, self.denominator)

    @property
    def _lilypond_format(self):
        if self.suppress:
            return []
        elif self.partial is None:
            return r'\time {}/{}'.format(self.numerator, self.denominator)
        else:
            result = []
            duration_string = self.partial.lilypond_duration_string
            partial_directive = r'\partial {}'.format(duration_string)
            result.append(partial_directive)
            string =r'\time {}/{}'.format(self.numerator, self.denominator)
            result.append(string)
            return result

    @property
    def _storage_format_specification(self):
        from abjad.tools import systemtools
        return systemtools.StorageFormatSpecification(
            self,
            keyword_argument_names=(
                'partial',
                'suppress',
                ),
            positional_argument_values=(
                self.pair,
                ),
            )

    ### PUBLIC PROPERTIES ###

    @property
    def denominator(self):
        r'''Time signature denominator.

        ::

            >>> time_signature.denominator
            8

        Returns positive integer.
        '''
        return self._denominator

    @property
    def duration(self):
        r'''Time signature duration.

        ::

            >>> TimeSignature((3, 8)).duration
            Duration(3, 8)

        Returns duration.
        '''
        return durationtools.Duration(self.numerator, self.denominator)

    @property
    def has_non_power_of_two_denominator(self):
        r'''True when time signature has non-power-of-two denominator.

        ::

            >>> time_signature = TimeSignature((7, 12))
            >>> time_signature.has_non_power_of_two_denominator
            True

        Otherwise false:

        ::

            >>> time_signature = TimeSignature((3, 8))
            >>> time_signature.has_non_power_of_two_denominator
            False

        Returns boolean.
        '''
        return self._has_non_power_of_two_denominator

    @property
    def implied_prolation(self):
        '''Time signature implied prolation.

        ..  container:: example

            **Example 1.** Implied prolation of time signature 
            with power-of-two denominator:

            ::

                >>> TimeSignature((3, 8)).implied_prolation
                Multiplier(1, 1)

        ..  container:: example

            **Example 2.** Implied prolation of time signature 
            with non-power-of-two denominator:

            ::

                >>> TimeSignature((7, 12)).implied_prolation
                Multiplier(2, 3)

        Returns multiplier.
        '''
        dummy_duration = durationtools.Duration(1, self.denominator)
        return dummy_duration.implied_prolation

    @property
    def numerator(self):
        r'''Time signature nuemrator.

        ::

            >>> time_signature.numerator
            3

        Returns positive integer.
        '''
        return self._numerator

    @property
    def pair(self):
        '''Time signature numerator / denominator pair.

        ::

            >>> TimeSignature((3, 8)).pair
            (3, 8)

        Returns pair.
        '''
        return (self.numerator, self.denominator)

    @property
    def partial(self):
        r'''Duration of time signature pick-up.

        ::

            >>> time_signature.partial

        Returns duration or none.
        '''
        return self._partial

    ### PUBLIC METHODS ###

    def with_power_of_two_denominator(
        self, 
        contents_multiplier=durationtools.Multiplier(1),
        ):
        r'''Creates new time signature equivalent to current
        time signature with power-of-two denominator.

            >>> time_signature = TimeSignature((3, 12))

        ::

            >>> time_signature.with_power_of_two_denominator()
            TimeSignature((2, 8))

        Returns new time signature.
        '''
        # check input
        contents_multiplier = durationtools.Multiplier(contents_multiplier)

        # save non_power_of_two time_signature and denominator
        non_power_of_two_denominator = self.denominator

        # find power_of_two denominator
        if contents_multiplier == durationtools.Multiplier(1):
            power_of_two_denominator = \
                mathtools.greatest_power_of_two_less_equal(
                non_power_of_two_denominator)
        else:
            power_of_two_denominator = \
                mathtools.greatest_power_of_two_less_equal(
                non_power_of_two_denominator, 1)

        # find power_of_two pair
        non_power_of_two_pair = mathtools.NonreducedFraction(self.pair)
        power_of_two_pair = non_power_of_two_pair.with_denominator(
            power_of_two_denominator)

        # return new power_of_two time signature
        return type(self)(power_of_two_pair)



# -*- encoding: utf-8 -*-
from abjad import *


def test_scoretools_make_spacer_skip_measures_01():
    r'''Make list of skip-populated measures.
    '''

    staff = Staff(scoretools.make_spacer_skip_measures([(1, 8), (5, 16), (5, 16), (1, 4)]))
    scoretools.set_always_format_time_signature_of_measures_in_expr(staff)

    r'''
    \new Staff {
        {
            \time 1/8
            s1 * 1/8
        }
        {
            \time 5/16
            s1 * 5/16
        }
        {
            \time 5/16
            s1 * 5/16
        }
        {
            \time 1/4
            s1 * 1/4
        }
    }
    '''

    assert systemtools.TestManager.compare(
        staff,
        r'''
        \new Staff {
            {
                \time 1/8
                s1 * 1/8
            }
            {
                \time 5/16
                s1 * 5/16
            }
            {
                \time 5/16
                s1 * 5/16
            }
            {
                \time 1/4
                s1 * 1/4
            }
        }
        '''
        )



# -*- encoding: utf-8 -*-
import copy
from abjad.tools.abctools.AbjadObject import AbjadObject


class LilyPondComment(AbjadObject):
    r'''A LilyPond comment.

    ::

        >>> note = Note("c'4")
        >>> comment = indicatortools.LilyPondComment('this is a comment')
        >>> attach(comment, note)
        >>> show(note) # doctest: +SKIP

    ..  doctest::

        >>> print format(note)
        % this is a comment
        c'4

    Initializes LilyPond comment from contents string;
    or contents string and format slot;
    or from other LilyPond comment;
    or from other LilyPond comment and format slot.
    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        '_contents_string',
        '_format_leaf_children',
        '_format_slot',
        )

    _format_leaf_children = False

    ### INITIALIZER ###

    def __init__(self, *args):
        if len(args) == 1 and isinstance(args[0], type(self)):
            contents_string = copy.copy(args[0].contents_string)
            format_slot = copy.copy(args[0].format_slot)
        elif len(args) == 1 and not isinstance(args[0], type(self)):
            contents_string = copy.copy(args[0])
            format_slot = None
        elif len(args) == 2 and isinstance(args[0], type(self)):
            contents_string = copy.copy(args[0].contents_string)
            format_slot = args[1]
        elif len(args) == 2 and not isinstance(args[0], type(self)):
            contents_string = args[0]
            format_slot = args[1]
        elif len(args) == 0:
            contents_string = 'foo'
            format_slot = None
        else:
            message = 'can not initialize {} from {!r}.'
            message = message.format(type(self).__name__, args)
            raise ValueError(message)
        format_slot = format_slot or 'before'
        self._contents_string = contents_string
        self._format_slot = format_slot

    ### SPECIAL METHODS ###

    def __copy__(self, *args):
        r'''Copies LilyPond comment.

        Returns new LilyPond comment.
        '''
        new = type(self)(self._contents_string)
        new.format_slot = self.format_slot
        return new

    def __eq__(self, arg):
        r'''True when `arg` is a LilyPond comment with contents string
        equal to LilyPond comment. Otherwise false.

        Returns boolean.
        '''
        if isinstance(arg, type(self)):
            return self._contents_string == arg._contents_string
        return False

    ### PRIVATE PROPERTIES ###

    @property
    def _contents_repr_string(self):
        return repr(self._contents_string)

    @property
    def _lilypond_format(self):
        from abjad.tools import stringtools
        command = stringtools.snake_case_to_lower_camel_case(
            self.contents_string)
        return r'%% %s' % command

    @property
    def _storage_format_specification(self):
        from abjad.tools import systemtools
        positional_argument_values = [self.contents_string]
        if self.format_slot is not None:
            positional_argument_values.append(self.format_slot)
        return systemtools.StorageFormatSpecification(
            self,
            positional_argument_values=positional_argument_values,
            )

    ### PUBLIC PROPERTIES ###

    @property
    def contents_string(self):
        r'''Contents string of LilyPond comment.

        ::

            >>> comment.contents_string
            'this is a comment'

        Returns string.
        '''
        return self._contents_string

    @property
    def format_slot(self):
        r'''Format slot of LilyPond comment.

        ::

            >>> comment.format_slot
            'before'

        Returns string.
        '''
        return self._format_slot



# -*- encoding: utf-8 -*-


def permute_named_pitch_carrier_list_by_twelve_tone_row(pitches, row):
    '''Permute named pitch carrier list by twelve-tone `row`:

    ::

        >>> notes = scoretools.make_notes([17, -10, -2, 11], [Duration(1, 4)])
        >>> row = pitchtools.TwelveToneRow([10, 0, 2, 6, 8, 7, 5, 3, 1, 9, 4, 11])
        >>> pitchtools.permute_named_pitch_carrier_list_by_twelve_tone_row(notes, row)
        [Note('bf4'), Note('d4'), Note("f''4"), Note("b'4")]

    Function works by reference only. No objects are copied.

    Returns list.
    '''
    from abjad.tools import pitchtools
    from abjad.tools import scoretools

    if not isinstance(row, pitchtools.TwelveToneRow):
        message = 'must be twelve-tone row: {!r}.'.format(row)
        raise TypeError(message)

    result = []

    for pc in row:
        matching_pitches = []
        for pitch in pitches:
            if isinstance(pitch, pitchtools.NamedPitch):
                if pitch.numbered_pitch_class == pc:
                    matching_pitches.append(pitch)
            elif isinstance(pitch, scoretools.Note):
                if pitchtools.NumberedPitchClass(pitch.written_pitch) == pc:
                    matching_pitches.append(pitch)
            else:
                message = 'must be pitch or note: {!r}'.format(pitch)
                raise TypeError(message)
        result.extend(matching_pitches)

    return result



# -*- encoding: utf-8 -*-
r'''Module: rhythmmakertools.

The following name changes were made when migrating into Abjad for initial check-in:

    NoteFilledTokens ==> NoteRhythmMaker
    RestFilledTokens ==> RestRhythmMaker
    PatternedTokens ==> TaleaRhythmMaker

    PartForcedChunkWithPatternedTokens ==> OutputBurnishedTaleaRhythmMaker
    PartForcedPatternedTokens ==> DivisionBurnishedTaleaRhythmMaker

    SignalAffixedChunkWithNoteFilledTokens ==> OutputIncisedNoteRhythmMaker
    SignalAffixedChunkWithRestFilledTokens ==> OutputIncisedtRestFilledTimeTokenMaker
    SignalAffixedNoteFilledTokens ==> DivisionIncisedNoteRhythmMaker
    SignalAffixedRestFilledTokens ==> DivisionIncisedRestRhythmMaker

    _PartForcedObjectWithPatternedTokens ==> BurnishedRhythmMaker
    _RhythmicKaleid ==> RhythmMaker
    _SignalAffixedChunkWithFilledTokens ==> OutputIncisedRhythmMaker
    _SignalAffixedFilledTokens ==> DivisionIncisedRhythmMaker
    _SignalAffixedObjectWithFilledTokens ==> IncisedRhythmMaker

These name changes no longer matter when using the public version of the package.
'''
from abjad.tools import systemtools

systemtools.ImportManager.import_structured_package(
	__path__[0],
	globals(),
	)

_documentation_section = 'core'



# -*- encoding: utf-8 -*-
from abjad import *


def test_sequencetools_get_indices_of_sequence_elements_equal_to_true_01():

    sequence_1 = [0, 0, 0, 1, 1, 1, 0, 0, 0, 1, 1, 1]
    sequence_2 = sequencetools.get_indices_of_sequence_elements_equal_to_true(sequence_1)
    assert sequence_2 == (3, 4, 5, 9, 10, 11)


def test_sequencetools_get_indices_of_sequence_elements_equal_to_true_02():

    sequence_1 = [0, 0, 0, 0, 0, 0]
    sequence_2 = sequencetools.get_indices_of_sequence_elements_equal_to_true(sequence_1)
    assert sequence_2 == ()



# -*- encoding: utf-8 -*-
import time
from abjad.tools.abctools import ContextManager


class Timer(ContextManager):
    r'''A timing context manager:

    ::

        >>> timer = systemtools.Timer()
        >>> with timer:
        ...     for _ in xrange(1000000):
        ...         x = 1 + 1
        ...
        >>> timer.elapsed_time # doctest: +SKIP
        0.092742919921875

    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        '_start_time',
        '_stop_time',
        )

    ### INITIALIZER ###

    def __init__(self):
        self._start_time = None
        self._stop_time = None

    ### SPECIAL METHODS ###

    def __enter__(self):
        r'''Enters context manager.

        Returns none.
        '''
        self._stop_time = None
        self._start_time = time.time()

    def __exit__(self, exc_type, exc_value, traceback):
        r'''Exist context manager.

        Returns none.
        '''
        self._stop_time = time.time()

    ### PUBLIC PROPERTIES ###

    @property
    def elapsed_time(self):
        r'''Elapsed time.

        Return float or none.
        '''
        if self.stop_time is not None and self.start_time is not None:
            return self.stop_time - self.start_time
        return None

    @property
    def start_time(self):
        r'''Start time of timer.

        Returns time.
        '''
        return self._start_time

    @property
    def stop_time(self):
        r'''Stop time of timer.

        Returns time.
        '''
        return self._stop_time



# -*- encoding: utf-8 -*-
from abjad import *
import pytest


def test_scoretools_NoteHeadInventory_get_01():
    r'''Get note heads by pitch number.
    '''

    chord = Chord("<c' d' b'>4")

    note_head = chord.note_heads.get(0)
    assert note_head.written_pitch == 0

    note_head = chord.note_heads.get(2)
    assert note_head.written_pitch == 2

    note_head = chord.note_heads.get(11)
    assert note_head.written_pitch == 11


def test_scoretools_NoteHeadInventory_get_02():
    r'''Get note heads by LilyPond input string.
    '''

    chord = Chord("<c' d' b'>4")

    note_head = chord.note_heads.get("c'")
    assert note_head.written_pitch == 0

    note_head = chord.note_heads.get("d'")
    assert note_head.written_pitch == 2

    note_head = chord.note_heads.get("b'")
    assert note_head.written_pitch == 11


def test_scoretools_NoteHeadInventory_get_03():
    r'''Get note heads by pitch.
    '''

    chord = Chord("<c' d' b'>4")

    pitch = NamedPitch("c'")
    note_head = chord.note_heads.get(pitch)
    assert note_head.written_pitch == 0

    pitch = NamedPitch("d'")
    note_head = chord.note_heads.get(pitch)
    assert note_head.written_pitch == 2

    pitch = NamedPitch("b'")
    note_head = chord.note_heads.get(pitch)
    assert note_head.written_pitch == 11


def test_scoretools_NoteHeadInventory_get_04():
    '''Raise exceptions when chord has too few or too many note heads.
    '''

    chord = Chord("<c' d' d'>4")

    assert pytest.raises(Exception, 'chord.note_heads.get(9)')
    assert pytest.raises(Exception, 'chord.note_heads.get(2)')



# -*- encoding: utf-8 -*-
from abjad import *


def test_indicatortools_Clef_middle_c_position_01():

    assert Clef('treble').middle_c_position == -6
    assert Clef('alto').middle_c_position == 0
    assert Clef('tenor').middle_c_position == 2
    assert Clef('bass').middle_c_position == 6
    assert Clef('treble^8').middle_c_position == -13
    assert Clef('alto^15').middle_c_position == -13
    assert Clef('tenor_8').middle_c_position == 9
    assert Clef('bass_15').middle_c_position == 19
